@@ -1,20 +1,24 @@
+from typing import Tuple
+
 import numpy as np
 import scipy.optimize as sciopt
+from numpy import float64, ndarray
 
 
-def gaussian(x, *p):
+def gaussian(x: ndarray, *p) -> ndarray:
     A, mu, sigma = p
     return A * np.exp(-((x - mu) ** 2) / (2.0 * sigma ** 2))
 
 
-def fit_gaussian(x, y, z_2d, save_fits=False):
-    """ fit a gaussian and
+def fit_gaussian(
+    x: ndarray, y: ndarray, z_2d: ndarray, save_fits: bool = False
+) -> Tuple[float64, Tuple[float64, float64], ndarray]:
+    """fit a gaussian
 
     Returns:
-
-    - effective area
-    - center of the mode
-    - Sigma^2
+        effective area
+        center of the mode
+        Sigma^2
     """
     z = z_2d
 
