@@ -9,6 +9,7 @@ from modes._structure import RidgeWaveguide
 from modes.autoname import autoname, clean_value
 from modes.get_modes_jsonpath import get_modes_jsonpath
 from modes.materials import nitride, sio2
+from modes.types import Field
 from modes.waveguide import waveguide
 
 
@@ -48,13 +49,13 @@ def _full(
     **wg_kwargs
 ) -> ModeSolverFullyVectorial:
     """Returns mode solver and
-    writes waveguide material index
+    can also plot the index profile.
 
     Args:
         n_modes: 2
         wg: waveguide object
+        plot_index: plot index profile
         wg_kwargs: for waveguide
-        plot: plot modes
     """
 
     wg = wg or waveguide(**wg_kwargs)
@@ -71,10 +72,10 @@ def mode_solver_full(
     n_modes: int = 2,
     overwrite: bool = False,
     plot: bool = False,
-    plot_index: bool = True,
+    plot_index: bool = False,
     logscale: bool = False,
     wg: Optional[RidgeWaveguide] = None,
-    fields_to_write: Tuple[str, ...] = (
+    fields_to_write: Tuple[Field, ...] = (
         "Ex",
         "Ey",
         "Ez",
@@ -93,7 +94,7 @@ def mode_solver_full(
         plot_index: plots index profile
         logscale: plots mode in logscale
         wg: waveguide
-        fields_to_write:
+        fields_to_write: List of fields_to_write "Ex", "Ey", "Ez", "Hx", "Hy", "Hz"
         x_step: 0.02 grid step (um)
         y_step: 0.02 grid step (um)
         thickness: 0.22 (um)
