@@ -1,12 +1,17 @@
+from typing import Dict, List, Union
+
 import matplotlib.pylab as plt
 import numpy as np
 import pytest
 import tqdm
+from numpy import float64, ndarray
 
 from modes.mode_solver_full import mode_solver_full
 
 
-def sweep_wavelength(wavelengths, plot=True, overwrite=False, **wg_kwargs):
+def sweep_wavelength(
+    wavelengths: ndarray, plot: bool = True, overwrite: bool = False, **wg_kwargs
+) -> Dict[str, Union[List[ndarray], List[List[float64]], List[List[float]]]]:
     """
 
     Solve for the effective indices of a fixed structure at
@@ -18,7 +23,7 @@ def sweep_wavelength(wavelengths, plot=True, overwrite=False, **wg_kwargs):
         overwrite: when True forces to resimulate the structure
 
     Returns:
-        wg_args: arguments for the waveguide
+        wg_kwargs: arguments for the waveguide
         results: dict of results
         resuls['n_effs']: A list of the effective indices found for each wavelength.
         resuls['fractions_te']:
@@ -51,7 +56,11 @@ def sweep_wavelength(wavelengths, plot=True, overwrite=False, **wg_kwargs):
     if plot:
         title = "$n_{eff}$ vs Wavelength"
         ms._plot_n_effs(
-            filename_neffs, filename_fraction_te, "Wavelength", "n_{eff}", title,
+            filename_neffs,
+            filename_fraction_te,
+            "Wavelength",
+            "n_{eff}",
+            title,
         )
         plt.ylabel("$n_{eff}$")
 
@@ -60,7 +69,7 @@ def sweep_wavelength(wavelengths, plot=True, overwrite=False, **wg_kwargs):
 
 
 @pytest.mark.parametrize("overwrite", [True, False])
-def test_sweep(overwrite):
+def test_sweep(overwrite: bool) -> None:
     wavelengths = np.arange(1.30, 1.60, 0.1)
     r = sweep_wavelength(wavelengths=wavelengths, overwrite=overwrite)
     print(r["n_effs"][0])
